﻿using System;
using System.Collections;
using Alachisoft.NCache.Runtime.Caching;
using Alachisoft.NCache.Web.Caching;
using Alachisoft.NCache.sample.data;

// ===============================================================================
// Alachisoft (R) NCache Sample Code.
// NCache Object Query Language sample
// ===============================================================================
// Copyright © Alachisoft.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
// ===============================================================================


namespace ObjectQueryLanguage
{
	public class ObjectQueryLanguage
	{

		public static void Main(string[] args)
		{
			try
			{
				//Initializing cache
				Cache cache;
                cache = NCache.InitializeCache("mypartitionedcache");
				cache.Clear();

				//Querying an item in NCache requires either NamedTags or
				//Indexes defined for objects or both.

				//Querying a simple cache item via namedTags

				NamedTagsDictionary nameTagDict = new NamedTagsDictionary();

				nameTagDict.Add("Employee", "DavidBrown");
				nameTagDict.Add("Department", "Mathematics");

				cache.Add("Employee:DavidBrown", "Department:Mathematics", nameTagDict);

				//Defining Searching criteria
				Hashtable criteria = new Hashtable();
				criteria["Employee"] = "DavidBrown";

				string query = "SELECT System.String WHERE this.Employee = ?";

				IDictionary result = cache.SearchEntries(query, criteria);

				if (result.Count > 0)
				{
					IEnumerator itor = result.Values.GetEnumerator();
					while (itor.MoveNext())
					{
						Console.WriteLine(itor.Current);
					}
				}

				Console.WriteLine();

				//New product object whose index is definex in NCache Manager
				Product product = new Product(1, "UninterruptedPowerSupply", "ClassA", "QCPassed");

				cache.Add("Product:UPS", product);

				//  Query can only be applied to C# Primitive data types:	
				//  and
				//  for those non primitive data types whose indexes are defined 
				//  in NCache manager

                query = "Select Alachisoft.NCache.sample.data.Product Where this.productId = ?";
				criteria = new Hashtable();
				criteria["productId"] = 1;

				result = cache.SearchEntries(query, criteria);

				if (result.Count > 0)
				{
					IEnumerator itor = result.Values.GetEnumerator();
					while (itor.MoveNext())
					{
						Product productFound = (Product) itor.Current;
						printProductDetails(productFound);
					}
				}

				//Querying indexed object via NamedTags

				nameTagDict = new NamedTagsDictionary();
                nameTagDict.Add("PName", "UninterruptedPowerSupply");
				nameTagDict.Add("Class", "ClassA");

				cache.Insert("Product:UPS", product, nameTagDict);

                query = "Select Alachisoft.NCache.sample.data.Product Where this.Name = ?";

				criteria = new Hashtable();
				criteria["Name"] = "UninterruptedPowerSupply";

				result = cache.SearchEntries(query, criteria);

				if (result.Count > 0)
				{
					IEnumerator itor = result.Values.GetEnumerator();
					while (itor.MoveNext())
					{
						Product productFound = (Product) itor.Current;
						printProductDetails(productFound);
					}
				}

				//Must dispose cache
				cache.Dispose();

				Environment.Exit(0);

			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
				Environment.Exit(0);
			}
		}
        /// <summary>
        /// Method for printing details of product type.
        /// </summary>
        /// <param name="product"></param>
		internal static void printProductDetails(Product product)
		{
			Console.WriteLine("Id:       " + product.Id);
			Console.WriteLine("Name:     " + product.Name);
			Console.WriteLine("Class:    " + product.ClassName);
			Console.WriteLine("Category: " + product.Category);
			Console.WriteLine();
		}
	}

}